package com.sun.org.apache.xml.internal.security.signature;

import com.sun.org.slf4j.internal.LoggerFactory;
import javax.crypto.SecretKey;
import com.sun.org.apache.xml.internal.security.keys.content.X509Data;
import com.sun.org.apache.xml.internal.security.transforms.Transforms;
import java.security.cert.X509Certificate;
import com.sun.org.apache.xml.internal.security.utils.resolver.ResourceResolverSpi;
import com.sun.org.apache.xml.internal.security.utils.resolver.ResourceResolver;
import com.sun.org.apache.xml.internal.security.algorithms.SignatureAlgorithm;
import com.sun.org.apache.xml.internal.security.utils.I18n;
import java.io.IOException;
import com.sun.org.apache.xml.internal.security.c14n.InvalidCanonicalizerException;
import com.sun.org.apache.xml.internal.security.c14n.CanonicalizationException;
import com.sun.org.apache.xml.internal.security.utils.UnsyncBufferedOutputStream;
import com.sun.org.apache.xml.internal.security.utils.SignerOutputStream;
import java.security.PublicKey;
import java.security.Key;
import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import com.sun.org.apache.xml.internal.security.utils.XMLUtils;
import com.sun.org.apache.xml.internal.security.utils.ElementProxy;
import com.sun.org.apache.xml.internal.security.exceptions.XMLSecurityException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import com.sun.org.apache.xml.internal.security.keys.KeyInfo;
import com.sun.org.slf4j.internal.Logger;
import com.sun.org.apache.xml.internal.security.utils.SignatureElementProxy;

public final class XMLSignature extends SignatureElementProxy
{
    public static final String ALGO_ID_MAC_HMAC_SHA1 = "http://www.w3.org/2000/09/xmldsig#hmac-sha1";
    public static final String ALGO_ID_SIGNATURE_DSA = "http://www.w3.org/2000/09/xmldsig#dsa-sha1";
    public static final String ALGO_ID_SIGNATURE_DSA_SHA256 = "http://www.w3.org/2009/xmldsig11#dsa-sha256";
    public static final String ALGO_ID_SIGNATURE_RSA = "http://www.w3.org/2000/09/xmldsig#rsa-sha1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA1 = "http://www.w3.org/2000/09/xmldsig#rsa-sha1";
    public static final String ALGO_ID_SIGNATURE_NOT_RECOMMENDED_RSA_MD5 = "http://www.w3.org/2001/04/xmldsig-more#rsa-md5";
    public static final String ALGO_ID_SIGNATURE_RSA_RIPEMD160 = "http://www.w3.org/2001/04/xmldsig-more#rsa-ripemd160";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA224 = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha224";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA256 = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA384 = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha384";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA512 = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA1_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha1-rsa-MGF1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA224_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha224-rsa-MGF1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA256_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha256-rsa-MGF1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA384_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha384-rsa-MGF1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA512_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha512-rsa-MGF1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA3_224_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha3-224-rsa-MGF1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA3_256_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha3-256-rsa-MGF1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA3_384_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha3-384-rsa-MGF1";
    public static final String ALGO_ID_SIGNATURE_RSA_SHA3_512_MGF1 = "http://www.w3.org/2007/05/xmldsig-more#sha3-512-rsa-MGF1";
    public static final String ALGO_ID_MAC_HMAC_NOT_RECOMMENDED_MD5 = "http://www.w3.org/2001/04/xmldsig-more#hmac-md5";
    public static final String ALGO_ID_MAC_HMAC_RIPEMD160 = "http://www.w3.org/2001/04/xmldsig-more#hmac-ripemd160";
    public static final String ALGO_ID_MAC_HMAC_SHA224 = "http://www.w3.org/2001/04/xmldsig-more#hmac-sha224";
    public static final String ALGO_ID_MAC_HMAC_SHA256 = "http://www.w3.org/2001/04/xmldsig-more#hmac-sha256";
    public static final String ALGO_ID_MAC_HMAC_SHA384 = "http://www.w3.org/2001/04/xmldsig-more#hmac-sha384";
    public static final String ALGO_ID_MAC_HMAC_SHA512 = "http://www.w3.org/2001/04/xmldsig-more#hmac-sha512";
    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA1 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha1";
    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA224 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha224";
    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA256 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha256";
    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA384 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha384";
    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA512 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha512";
    public static final String ALGO_ID_SIGNATURE_ECDSA_RIPEMD160 = "http://www.w3.org/2007/05/xmldsig-more#ecdsa-ripemd160";
    private static final Logger LOG;
    private SignedInfo signedInfo;
    private KeyInfo keyInfo;
    private boolean followManifestsDuringValidation;
    private Element signatureValueElement;
    private static final int MODE_SIGN = 0;
    private static final int MODE_VERIFY = 1;
    private int state;
    
    public XMLSignature(Document document, String s, String s2) throws XMLSecurityException {
        this(document, s, s2, 0, "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");
    }
    
    public XMLSignature(Document document, String s, String s2, int n) throws XMLSecurityException {
        this(document, s, s2, n, "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");
    }
    
    public XMLSignature(Document document, String s, String s2, String s3) throws XMLSecurityException {
        this(document, s, s2, 0, s3);
    }
    
    public XMLSignature(Document doc, String baseURI, String s, int n, String s2) throws XMLSecurityException {
        super(doc);
        this.followManifestsDuringValidation = false;
        this.state = 0;
        String defaultPrefix = ElementProxy.getDefaultPrefix("http://www.w3.org/2000/09/xmldsig#");
        if (defaultPrefix == null || defaultPrefix.length() == 0) {
            this.getElement().setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns", "http://www.w3.org/2000/09/xmldsig#");
        }
        else {
            this.getElement().setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:" + defaultPrefix, "http://www.w3.org/2000/09/xmldsig#");
        }
        this.addReturnToSelf();
        this.baseURI = baseURI;
        this.appendSelf(this.signedInfo = new SignedInfo(this.getDocument(), s, n, s2));
        this.addReturnToSelf();
        this.appendSelf(this.signatureValueElement = XMLUtils.createElementInSignatureSpace(this.getDocument(), "SignatureValue"));
        this.addReturnToSelf();
    }
    
    public XMLSignature(Document doc, String baseURI, Element element, Element element2) throws XMLSecurityException {
        super(doc);
        this.followManifestsDuringValidation = false;
        this.state = 0;
        String defaultPrefix = ElementProxy.getDefaultPrefix("http://www.w3.org/2000/09/xmldsig#");
        if (defaultPrefix == null || defaultPrefix.length() == 0) {
            this.getElement().setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns", "http://www.w3.org/2000/09/xmldsig#");
        }
        else {
            this.getElement().setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:" + defaultPrefix, "http://www.w3.org/2000/09/xmldsig#");
        }
        this.addReturnToSelf();
        this.baseURI = baseURI;
        this.appendSelf(this.signedInfo = new SignedInfo(this.getDocument(), element, element2));
        this.addReturnToSelf();
        this.appendSelf(this.signatureValueElement = XMLUtils.createElementInSignatureSpace(this.getDocument(), "SignatureValue"));
        this.addReturnToSelf();
    }
    
    public XMLSignature(Element element, String s) throws XMLSignatureException, XMLSecurityException {
        this(element, s, true);
    }
    
    public XMLSignature(Element element, String s, boolean secureValidation) throws XMLSignatureException, XMLSecurityException {
        super(element, s);
        this.followManifestsDuringValidation = false;
        this.state = 0;
        Element nextElement = XMLUtils.getNextElement(element.getFirstChild());
        if (nextElement == null) {
            throw new XMLSignatureException("xml.WrongContent", new Object[] { "SignedInfo", "Signature" });
        }
        this.signedInfo = new SignedInfo(nextElement, s, secureValidation);
        this.signatureValueElement = XMLUtils.getNextElement(XMLUtils.getNextElement(element.getFirstChild()).getNextSibling());
        if (this.signatureValueElement == null) {
            throw new XMLSignatureException("xml.WrongContent", new Object[] { "SignatureValue", "Signature" });
        }
        Attr attributeNodeNS = this.signatureValueElement.getAttributeNodeNS(null, "Id");
        if (attributeNodeNS != null) {
            this.signatureValueElement.setIdAttributeNode(attributeNodeNS, true);
        }
        Element nextElement2 = XMLUtils.getNextElement(this.signatureValueElement.getNextSibling());
        if (nextElement2 != null && "http://www.w3.org/2000/09/xmldsig#".equals(nextElement2.getNamespaceURI()) && "KeyInfo".equals(nextElement2.getLocalName())) {
            (this.keyInfo = new KeyInfo(nextElement2, s)).setSecureValidation(secureValidation);
        }
        for (Element element2 = XMLUtils.getNextElement(this.signatureValueElement.getNextSibling()); element2 != null; element2 = XMLUtils.getNextElement(element2.getNextSibling())) {
            Attr attributeNodeNS2 = element2.getAttributeNodeNS(null, "Id");
            if (attributeNodeNS2 != null) {
                element2.setIdAttributeNode(attributeNodeNS2, true);
            }
            for (Node node = element2.getFirstChild(); node != null; node = node.getNextSibling()) {
                if (node.getNodeType() == 1) {
                    Element element3 = (Element)node;
                    String localName = element3.getLocalName();
                    if ("Manifest".equals(localName)) {
                        new Manifest(element3, s);
                    }
                    else if ("SignatureProperties".equals(localName)) {
                        new SignatureProperties(element3, s);
                    }
                }
            }
        }
        this.state = 1;
        return;
    }
    
    public void setId(String value) {
        if (value != null) {
            this.setLocalIdAttribute("Id", value);
        }
    }
    
    public String getId() {
        return this.getLocalAttribute("Id");
    }
    
    public SignedInfo getSignedInfo() {
        return this.signedInfo;
    }
    
    public byte[] getSignatureValue() throws XMLSignatureException {
        return XMLUtils.decode(XMLUtils.getFullTextChildrenFromNode(this.signatureValueElement));
    }
    
    private void setSignatureValueElement(byte[] bytes) {
        while (this.signatureValueElement.hasChildNodes()) {
            this.signatureValueElement.removeChild(this.signatureValueElement.getFirstChild());
        }
        String s = XMLUtils.encodeToString(bytes);
        if (s.length() > 76 && !XMLUtils.ignoreLineBreaks()) {
            s = "\n" + s + "\n";
        }
        this.signatureValueElement.appendChild(this.createText(s));
    }
    
    public KeyInfo getKeyInfo() {
        if (this.state == 0 && this.keyInfo == null) {
            this.keyInfo = new KeyInfo(this.getDocument());
            Element element = this.keyInfo.getElement();
            Element selectDsNode = XMLUtils.selectDsNode(this.getElement().getFirstChild(), "Object", 0);
            if (selectDsNode != null) {
                this.getElement().insertBefore(element, selectDsNode);
                XMLUtils.addReturnBeforeChild(this.getElement(), selectDsNode);
            }
            else {
                this.appendSelf(element);
                this.addReturnToSelf();
            }
        }
        return this.keyInfo;
    }
    
    public void appendObject(ObjectContainer toAppend) throws XMLSignatureException {
        this.appendSelf(toAppend);
        this.addReturnToSelf();
    }
    
    public ObjectContainer getObjectItem(int number) {
        Element selectDsNode = XMLUtils.selectDsNode(this.getFirstChild(), "Object", number);
        try {
            return new ObjectContainer(selectDsNode, this.baseURI);
        }
        catch (XMLSecurityException ex) {
            return null;
        }
    }
    
    public int getObjectLength() {
        return this.length("http://www.w3.org/2000/09/xmldsig#", "Object");
    }
    
    public void sign(Key signingKey) throws XMLSignatureException {
        if ((signingKey instanceof PublicKey)) {
            throw new IllegalArgumentException(I18n.translate("algorithms.operationOnlyVerification"));
        }
        SignedInfo signedInfo = this.getSignedInfo();
        SignatureAlgorithm signatureAlgorithm = signedInfo.getSignatureAlgorithm();
        try (final SignerOutputStream out = new SignerOutputStream(signatureAlgorithm);
             final UnsyncBufferedOutputStream unsyncBufferedOutputStream = new UnsyncBufferedOutputStream(out)) {
            signedInfo.generateDigestValues();
            signatureAlgorithm.initSign(signingKey);
            signedInfo.signInOctetStream(unsyncBufferedOutputStream);
            this.setSignatureValueElement(signatureAlgorithm.sign());
        }
        catch (XMLSignatureException ex) {
            throw ex;
        }
        catch (CanonicalizationException ex2) {
            throw new XMLSignatureException(ex2);
        }
        catch (InvalidCanonicalizerException ex3) {
            throw new XMLSignatureException(ex3);
        }
        catch (XMLSecurityException ex4) {
            throw new XMLSignatureException(ex4);
        }
        catch (IOException ex5) {
            throw new XMLSignatureException(ex5);
        }
        return;
    }
    
    public void addResourceResolver(ResourceResolver resourceResolver) {
        this.getSignedInfo().addResourceResolver(resourceResolver);
    }
    
    public void addResourceResolver(ResourceResolverSpi resourceResolverSpi) {
        this.getSignedInfo().addResourceResolver(resourceResolverSpi);
    }
    
    public boolean checkSignatureValue(X509Certificate x509Certificate) throws XMLSignatureException {
        if (x509Certificate != null) {
            return this.checkSignatureValue(x509Certificate.getPublicKey());
        }
        throw new XMLSignatureException("empty", new Object[] { "Didn't get a certificate" });
    }
    
    public boolean checkSignatureValue(Key verificationKey) throws XMLSignatureException {
        if (verificationKey == null) {
            throw new XMLSignatureException("empty", new Object[] { "Didn't get a key" });
        }
        try {
            SignedInfo signedInfo = this.getSignedInfo();
            SignatureAlgorithm signatureAlgorithm = signedInfo.getSignatureAlgorithm();
            XMLSignature.LOG.debug("signatureMethodURI = {}", new Object[] { signatureAlgorithm.getAlgorithmURI() });
            XMLSignature.LOG.debug("jceSigAlgorithm = {}", new Object[] { signatureAlgorithm.getJCEAlgorithmString() });
            XMLSignature.LOG.debug("jceSigProvider = {}", new Object[] { signatureAlgorithm.getJCEProviderName() });
            XMLSignature.LOG.debug("PublicKey = {}", new Object[] { verificationKey });
            byte[] signatureValue = null;
            try (final SignerOutputStream out = new SignerOutputStream(signatureAlgorithm);
                 final UnsyncBufferedOutputStream unsyncBufferedOutputStream = new UnsyncBufferedOutputStream(out)) {
                signatureAlgorithm.initVerify(verificationKey);
                signedInfo.signInOctetStream(unsyncBufferedOutputStream);
                signatureValue = this.getSignatureValue();
            }
            catch (IOException e) {
                XMLSignature.LOG.debug(e.getMessage(), e);
            }
            catch (XMLSecurityException ex) {
                throw ex;
            }
            if (!signatureAlgorithm.verify(signatureValue)) {
                XMLSignature.LOG.warn("Signature verification failed.");
                return false;
            }
            return signedInfo.verify(this.followManifestsDuringValidation);
        }
        catch (XMLSignatureException ex2) {
            throw ex2;
        }
        catch (XMLSecurityException ex3) {
            throw new XMLSignatureException(ex3);
        }
    }
    
    public void addDocument(String s, Transforms transforms, String s2, String s3, String s4) throws XMLSignatureException {
        this.signedInfo.addDocument(this.baseURI, s, transforms, s2, s3, s4);
    }
    
    public void addDocument(String s, Transforms transforms, String s2) throws XMLSignatureException {
        this.signedInfo.addDocument(this.baseURI, s, transforms, s2, null, null);
    }
    
    public void addDocument(String s, Transforms transforms) throws XMLSignatureException {
        this.signedInfo.addDocument(this.baseURI, s, transforms, "http://www.w3.org/2000/09/xmldsig#sha1", null, null);
    }
    
    public void addDocument(String s) throws XMLSignatureException {
        this.signedInfo.addDocument(this.baseURI, s, null, "http://www.w3.org/2000/09/xmldsig#sha1", null, null);
    }
    
    public void addKeyInfo(X509Certificate x509certificate) throws XMLSecurityException {
        X509Data x509data = new X509Data(this.getDocument());
        x509data.addCertificate(x509certificate);
        this.getKeyInfo().add(x509data);
    }
    
    public void addKeyInfo(PublicKey pk) {
        this.getKeyInfo().add(pk);
    }
    
    public SecretKey createSecretKey(byte[] array) {
        return this.getSignedInfo().createSecretKey(array);
    }
    
    public void setFollowNestedManifests(boolean followManifestsDuringValidation) {
        this.followManifestsDuringValidation = followManifestsDuringValidation;
    }
    
    @Override
    public String getBaseLocalName() {
        return "Signature";
    }
    
    static {
        LOG = LoggerFactory.getLogger(XMLSignature.class);
    }
}
